@@ -14,96 +14,97 @@ unyt_array class.
 # -----------------------------------------------------------------------------
 
 import copy
-
+import re
 from functools import lru_cache
 from numbers import Number as numeric_type
+
 import numpy as np
 from numpy import (
-    add,
-    subtract,
-    multiply,
-    divide,
-    logaddexp,
-    logaddexp2,
-    true_divide,
-    floor_divide,
-    negative,
-    power,
-    remainder,
-    mod,
     absolute,
-    rint,
-    sign,
-    conj,
-    exp,
-    exp2,
-    log,
-    log2,
-    log10,
-    expm1,
-    log1p,
-    sqrt,
-    square,
-    reciprocal,
-    sin,
-    cos,
-    tan,
-    arcsin,
+    add,
     arccos,
+    arccosh,
+    arcsin,
+    arcsinh,
     arctan,
     arctan2,
-    hypot,
-    sinh,
-    cosh,
-    tanh,
-    arcsinh,
-    arccosh,
     arctanh,
-    deg2rad,
-    rad2deg,
     bitwise_and,
     bitwise_or,
     bitwise_xor,
-    invert,
-    left_shift,
-    right_shift,
-    greater,
-    greater_equal,
-    less,
-    less_equal,
-    not_equal,
+    ceil,
+    conj,
+    copysign,
+    cos,
+    cosh,
+    deg2rad,
+    divide,
+    divmod as divmod_,
     equal,
-    logical_and,
-    logical_or,
-    logical_xor,
-    logical_not,
-    maximum,
-    minimum,
+    exp,
+    exp2,
+    expm1,
+    fabs,
+    floor,
+    floor_divide,
     fmax,
     fmin,
-    isreal,
+    fmod,
+    frexp,
+    greater,
+    greater_equal,
+    heaviside,
+    hypot,
+    invert,
     iscomplex,
     isfinite,
     isinf,
     isnan,
-    signbit,
-    copysign,
-    nextafter,
-    modf,
-    ldexp,
-    frexp,
-    fmod,
-    floor,
-    ceil,
-    trunc,
-    fabs,
-    spacing,
-    positive,
-    divmod as divmod_,
     isnat,
-    heaviside,
-    ones_like,
+    isreal,
+    ldexp,
+    left_shift,
+    less,
+    less_equal,
+    log,
+    log1p,
+    log2,
+    log10,
+    logaddexp,
+    logaddexp2,
+    logical_and,
+    logical_not,
+    logical_or,
+    logical_xor,
     matmul,
+    maximum,
+    minimum,
+    mod,
+    modf,
+    multiply,
+    negative,
+    nextafter,
+    not_equal,
+    ones_like,
+    positive,
+    power,
+    rad2deg,
+    reciprocal,
+    remainder,
+    right_shift,
+    rint,
+    sign,
+    signbit,
+    sin,
+    sinh,
+    spacing,
+    sqrt,
+    square,
+    subtract,
+    tan,
+    tanh,
+    true_divide,
+    trunc,
 )
 from numpy.core.umath import _ones_like
 
@@ -111,40 +112,52 @@ try:
     from numpy.core.umath import clip
 except ImportError:
     clip = None
-from sympy import Rational
 import warnings
 
-from unyt.dimensions import angle, temperature
-from unyt.exceptions import (
-    IterableUnitCoercionError,
-    InvalidUnitEquivalence,
-    InvalidUnitOperation,
-    MKSCGSConversionError,
-    UnitOperationError,
-    UnitConversionError,
-    UnitsNotReducible,
-    SymbolNotFoundError,
-)
-from unyt.equivalencies import equivalence_registry
+from sympy import Rational
+
 from unyt._on_demand_imports import _astropy, _pint
 from unyt._pint_conversions import convert_pint_units
 from unyt._unit_lookup_table import default_unit_symbol_lut
-from unyt.unit_object import _check_em_conversion, _em_conversion, Unit
+from unyt.dimensions import angle, temperature
+from unyt.equivalencies import equivalence_registry
+from unyt.exceptions import (
+    InvalidUnitEquivalence,
+    InvalidUnitOperation,
+    IterableUnitCoercionError,
+    MKSCGSConversionError,
+    SymbolNotFoundError,
+    UnitConversionError,
+    UnitOperationError,
+    UnitsNotReducible,
+)
+from unyt.unit_object import Unit, _check_em_conversion, _em_conversion
 from unyt.unit_registry import (
-    _sanitize_unit_system,
     UnitRegistry,
-    default_unit_registry,
     _correct_old_unit_registry,
+    _sanitize_unit_system,
+    default_unit_registry,
 )
 
 NULL_UNIT = Unit()
-POWER_SIGN_MAPPING = {multiply: 1, divide: -1}
+POWER_MAPPING = {multiply: lambda x: x, divide: lambda x: 2 - x}
 
 __doctest_requires__ = {
     ("unyt_array.from_pint", "unyt_array.to_pint"): ["pint"],
     ("unyt_array.from_astropy", "unyt_array.to_astropy"): ["astropy"],
 }
 
+# This is partially adapted from the following SO thread
+# https://stackoverflow.com/questions/41668588/regex-to-match-scientific-notation
+_NUMB_PATTERN = r"^[+/-]?((?:0|[1-9]\d*)(?:\.\d*)?(?:[eE][+\-]?\d+)|\d*\.?\d+|\d+\.?\d*|nan\s|inf\s)"  # noqa: E501
+# *all* greek letters are considered valid unit string elements.
+# This may be an overshoot. We rely on unyt.Unit to do the actual validation
+_UNIT_PATTERN = r"([α-ωΑ-Ωa-zA-Z]+(\*\*([+/-]?[0-9]+)|[*/])?)+"
+_QUAN_PATTERN = r"{}\s*{}".format(_NUMB_PATTERN, _UNIT_PATTERN)
+_NUMB_REGEXP = re.compile(_NUMB_PATTERN)
+_UNIT_REGEXP = re.compile(_UNIT_PATTERN)
+_QUAN_REGEXP = re.compile(_QUAN_PATTERN)
+
 
 def _iterable(obj):
     try:
@@ -156,7 +169,7 @@ def _iterable(obj):
 
 @lru_cache(maxsize=128, typed=False)
 def _sqrt_unit(unit):
-    return 1, unit ** 0.5
+    return 1, unit**0.5
 
 
 @lru_cache(maxsize=128, typed=False)
@@ -192,7 +205,7 @@ def _preserve_units(unit1, unit2=None):
 
 @lru_cache(maxsize=128, typed=False)
 def _power_unit(unit, power):
-    return 1, unit ** power
+    return 1, unit**power
 
 
 @lru_cache(maxsize=128, typed=False)
@@ -211,7 +224,7 @@ def _divide_units(unit1, unit2):
 
 @lru_cache(maxsize=128, typed=False)
 def _reciprocal_unit(unit):
-    return 1, unit ** -1
+    return 1, unit**-1
 
 
 def _passthrough_unit(unit, unit2=None):
@@ -245,7 +258,13 @@ def _coerce_iterable_units(input_object, registry=None):
         if any([isinstance(o, unyt_array) for o in input_object]):
             ff = getattr(input_object[0], "units", NULL_UNIT)
             if any([ff != getattr(_, "units", NULL_UNIT) for _ in input_object]):
-                raise IterableUnitCoercionError(input_object)
+                ret = []
+                for datum in input_object:
+                    try:
+                        ret.append(datum.in_units(ff.units))
+                    except UnitConversionError:
+                        raise IterableUnitCoercionError(str(input_object))
+                return unyt_array(np.array(ret), ff, registry=registry)
             # This will create a copy of the data in the iterable.
             return unyt_array(np.array(input_object), ff, registry=registry)
     return np.asarray(input_object)
@@ -589,8 +608,7 @@ class unyt_array(np.ndarray):
         return str(self.view(np.ndarray)) + " " + str(self.units)
 
     def __format__(self, format_spec):
-        ret = super(unyt_array, self).__format__(format_spec)
-        return ret + " {}".format(self.units)
+        return "{} {}".format(self.d.__format__(format_spec), self.units)
 
     #
     # Start unit conversion methods
@@ -653,6 +671,12 @@ class unyt_array(np.ndarray):
                 # form, it's possible this may lose precision for very
                 # large integers
                 dsize = values.dtype.itemsize
+                if dsize == 1:
+                    raise ValueError(
+                        "Can't convert memory buffer in place. "
+                        f"Input dtype ({self.dtype}) has a smaller itemsize than the "
+                        "smallest floating point representation possible."
+                    )
                 new_dtype = "f" + str(dsize)
                 large = LARGE_INPUT.get(dsize, 0)
                 if large and np.any(np.abs(values) > large):
@@ -713,7 +737,7 @@ class unyt_array(np.ndarray):
         self.convert_to_units(
             self.units.get_base_equivalent(unit_system),
             equivalence=equivalence,
-            **kwargs
+            **kwargs,
         )
 
     def convert_to_cgs(self, equivalence=None, **kwargs):
@@ -833,7 +857,7 @@ class unyt_array(np.ndarray):
                 (conversion_factor, offset) = self.units.get_conversion_factor(
                     new_units, self.dtype
                 )
-            dsize = self.dtype.itemsize
+            dsize = max(2, self.dtype.itemsize)
             if self.dtype.kind in ("u", "i"):
                 large = LARGE_INPUT.get(dsize, 0)
                 if large and np.any(np.abs(self.d) > large):
@@ -843,8 +867,8 @@ class unyt_array(np.ndarray):
                         RuntimeWarning,
                         stacklevel=2,
                     )
-            new_dtype = np.dtype("f" + str(dsize))
-            conversion_factor = new_dtype.type(conversion_factor)
+            new_dtypekind = "c" if self.dtype.kind == "c" else "f"
+            new_dtype = np.dtype(new_dtypekind + str(dsize))
             ret = np.asarray(self.ndview * conversion_factor, dtype=new_dtype)
             if offset:
                 np.subtract(ret, offset, ret)
@@ -982,8 +1006,6 @@ class unyt_array(np.ndarray):
         else:
             to_units = self.units.get_base_equivalent(unit_system)
             conv, offset = self.units.get_conversion_factor(to_units, self.dtype)
-        new_dtype = np.dtype("f" + str(self.dtype.itemsize))
-        conv = new_dtype.type(conv)
         ret = self.v * conv
         if offset:
             ret = ret - offset
@@ -1207,8 +1229,8 @@ class unyt_array(np.ndarray):
         ----------
         arr : AstroPy Quantity
             The Quantity to convert from.
-        unit_registry : yt UnitRegistry, optional
-            A yt unit registry to use in the conversion. If one is not
+        unit_registry : unyt.UnitRegistry, optional
+            A unyt unit registry to use in the conversion. If one is not
             supplied, the default one will be used.
 
         Example
@@ -1265,8 +1287,8 @@ class unyt_array(np.ndarray):
         ----------
         arr : Pint Quantity
             The Quantity to convert from.
-        unit_registry : yt UnitRegistry, optional
-            A yt unit registry to use in the conversion. If one is not
+        unit_registry : unyt.UnitRegistry, optional
+            A unyt unit registry to use in the conversion. If one is not
             supplied, the default one will be used.
 
         Examples
@@ -1303,7 +1325,7 @@ class unyt_array(np.ndarray):
         unit_registry : Pint UnitRegistry, optional
             The Pint UnitRegistry to use in the conversion. If one is not
             supplied, the default one will be used. NOTE: This is not
-            the same as a yt UnitRegistry object.
+            the same as a unyt.UnitRegistry object.
 
         Examples
         --------
@@ -1327,6 +1349,52 @@ class unyt_array(np.ndarray):
         units = "*".join(units)
         return unit_registry.Quantity(self.value, units)
 
+    @staticmethod
+    def from_string(s, unit_registry=None):
+        """
+        Parse a string to a unyt_quantity object.
+
+        Parameters
+        ----------
+        s: str
+           A string representing a single quantity.
+        unit_registry: unyt.UnitRegistry, optional
+            A unyt unit registry to use in the conversion. If one is not
+            supplied, the default one will be used.
+
+        Examples
+        --------
+        >>> from unyt import unyt_quantity
+        >>> unyt_quantity.from_string("1cm")
+        unyt_quantity(1., 'cm')
+        >>> unyt_quantity.from_string("+1e3 Mearth")
+        unyt_quantity(1000., 'Mearth')
+        >>> unyt_quantity.from_string("-10. kg")
+        unyt_quantity(-10., 'kg')
+        >>> unyt_quantity.from_string(".66\tum")
+        unyt_quantity(0.66, 'μm')
+        >>> unyt_quantity.from_string("42")
+        unyt_quantity(42., '(dimensionless)')
+        >>> unyt_quantity.from_string("1.0 g/cm**3")
+        unyt_quantity(1., 'g/cm**3')
+        """
+        v = s.strip()
+        if re.fullmatch(_NUMB_REGEXP, v):
+            return float(re.match(_NUMB_REGEXP, v).group()) * Unit()
+        if re.fullmatch(_UNIT_REGEXP, v):
+            return 1 * Unit(re.match(_UNIT_REGEXP, v).group())
+        if not re.match(_QUAN_REGEXP, v):
+            raise ValueError("Received invalid quantity expression '{}'.".format(s))
+        res = re.search(_NUMB_REGEXP, v)
+        num = res.group()
+        res = re.search(_UNIT_REGEXP, v[res.span()[1] :])
+        unit = res.group()
+        return float(num) * Unit(unit, registry=unit_registry)
+
+    def to_string(self):
+        # this is implemented purely for symmetry's sake
+        return str(self)
+
     #
     # End unit conversion methods
     #
@@ -1358,9 +1426,10 @@ class unyt_array(np.ndarray):
         >>> a.write_hdf5('test_array_data.h5', dataset_name='dinosaurs',
         ...              info=myinfo)  # doctest: +SKIP
         """
-        from unyt._on_demand_imports import _h5py as h5py
         import pickle
 
+        from unyt._on_demand_imports import _h5py as h5py
+
         if info is None:
             info = {}
 
@@ -1418,9 +1487,10 @@ class unyt_array(np.ndarray):
             arrays are datasets at the top level by default.
 
         """
-        from unyt._on_demand_imports import _h5py as h5py
         import pickle
 
+        from unyt._on_demand_imports import _h5py as h5py
+
         if dataset_name is None:
             dataset_name = "array_data"
 
@@ -1624,14 +1694,29 @@ class unyt_array(np.ndarray):
 
     def __getitem__(self, item):
         ret = super(unyt_array, self).__getitem__(item)
-        if ret.shape == ():
-            return unyt_quantity(
-                ret, self.units, bypass_validation=True, name=self.name
-            )
+        if getattr(ret, "shape", None) == ():
+            ret = unyt_quantity(ret, bypass_validation=True, name=self.name)
+        try:
+            ret.units = self.units
+        except AttributeError:
+            pass
+        return ret
+
+    def __pow__(self, p):
+        """
+        Power function, over-rides the ufunc as
+        ``numpy`` passes the ``_ones_like`` ufunc for
+        powers of zero (in some cases), which leads to an
+        incorrect unit calculation.
+        """
+
+        if p == 0.0:
+            ret = self.ua
+            ret.units = Unit("dimensionless")
         else:
-            if hasattr(self, "units"):
-                ret.units = self.units
-            return ret
+            ret = self.__array_ufunc__(power, "__call__", self, p)
+
+        return ret
 
     #
     # Start operation methods
@@ -1676,11 +1761,11 @@ class unyt_array(np.ndarray):
                 # a reduction of a multiply or divide corresponds to
                 # a repeated product which we implement as an exponent
                 mul = 1
-                power_sign = POWER_SIGN_MAPPING[ufunc]
+                power_map = POWER_MAPPING[ufunc]
                 if "axis" in kwargs and kwargs["axis"] is not None:
-                    unit = u ** (power_sign * inp.shape[kwargs["axis"]])
+                    unit = u ** (power_map(inp.shape[kwargs["axis"]]))
                 else:
-                    unit = u ** (power_sign * inp.size)
+                    unit = u ** (power_map(inp.size))
             else:
                 # get unit of result
                 mul, unit = self._ufunc_registry[ufunc](u)
@@ -1821,8 +1906,10 @@ class unyt_array(np.ndarray):
             out_arr = np.array(out_arr, copy=False)
         elif ufunc in (modf, divmod_):
             out_arr = tuple((ret_class(o, unit) for o in out_arr))
-        elif out_arr.size == 1:
+        elif out_arr.shape == ():
             out_arr = unyt_quantity(np.asarray(out_arr), unit)
+        elif out_arr.size == 1:
+            out_arr = unyt_array(np.asarray(out_arr), unit)
         else:
             if ret_class is unyt_quantity:
                 # This happens if you do ndarray * unyt_quantity.
@@ -1900,7 +1987,7 @@ class unyt_array(np.ndarray):
         self.name = getattr(obj, "name", None)
 
     def __pos__(self):
-        """ Posify the data. """
+        """Posify the data."""
         # this needs to be defined for all numpy versions, see
         # numpy issue #9081
         return type(self)(super(unyt_array, self).__pos__(), self.units)
@@ -2068,6 +2155,19 @@ class unyt_quantity(unyt_array):
     def __round__(self):
         return type(self)(round(float(self)), self.units)
 
+    def reshape(self, *shape, order="C"):
+        # this is necessary to support some numpy operations
+        # natively, like numpy.meshgrid, which internally performs
+        # reshaping, e.g., arr.reshape(1, -1), which doesn't affect the size,
+        # but does change the object's internal representation to a >0D array
+        # see https://github.com/yt-project/unyt/issues/224
+        if len(shape) == 1:
+            shape = shape[0]
+        if shape == () or shape is None:
+            return super().reshape(shape, order=order)
+        else:
+            return unyt_array(self).reshape(shape, order=order)
+
 
 def _validate_numpy_wrapper_units(v, arrs):
     if not any(isinstance(a, unyt_array) for a in arrs):
@@ -2335,7 +2435,8 @@ def loadtxt(fname, dtype="float", delimiter="\t", usecols=None, comments="#"):
             # Here we catch the first line of numbers
             col_words = line.strip().split(delimiter)
             for word in col_words:
-                float(word)
+                # test that word can be converted to a number
+                complex(word)
             num_cols = len(col_words)
             break
     f.close()
